indexing
	description: "Objects that find program points of interest and process them"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_PROGRAM_POINT_FINDER

inherit
	CI_INSTRUCTION_COUNTER
		rename
			make as make_with_universe
		export {NONE}
			make_with_universe
		redefine
			process_class,
--			process_deferred_procedure,
--			process_deferred_function,
			process_do_procedure,
			process_do_function,
--			process_external_procedure,
--			process_external_function,
			process_once_procedure,
			process_once_function,
			process_loop_instruction
		end

	CI_SETTINGS_CLIENT

	CI_CODE_GENERATION
		export {NONE}
			all
		end

create
	make

feature -- Initialization
	make (a_universe: like universe; a_printer: like printer) is
			-- Create program point finder that works within `a_universe' with program point information printer `a_printer'
		require
			a_universe_exists: a_universe /= Void
			a_printer_exists: a_printer /= Void
		do
			make_with_universe (a_universe)
			printer := a_printer
			create program_points.make (Settings.classes_to_instrument.count * 20)  -- Approx 10 features a class (?)
			program_points.compare_objects
		ensure
			universe_set: universe = a_universe
			printer_set: printer = a_printer
		end

feature -- Access
	program_points: HASH_TABLE [CI_PROGRAM_POINT, STRING]
			-- All met program points

	printer: CI_PROGRAM_POINT_PRINTER
			-- Program point printer

feature -- Setting
	set_printer (a_printer: CI_PROGRAM_POINT_PRINTER) is
			-- Set `printer' to `a_printer'
		require
			a_printer_exist: a_printer /= Void
		do
			printer := a_printer
		ensure
			printer_set: printer = a_printer
		end

feature {ET_AST_NODE} -- Processing
	process_class (a_class: ET_CLASS) is
			-- Process child nodes, create and process object program point for `a_class'
		local
			stable_time: CI_STABLE_TIME
		do
			current_class := a_class
			Precursor (a_class)

			create stable_time.make (a_class, universe)
			process_program_point (stable_time)
		end

--	process_deferred_procedure (a_routine: ET_DEFERRED_PROCEDURE) is
--			-- Process child nodes, create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := a_routine
--			current_function := Void
--			process_procedure
--		end

--	process_deferred_function (a_routine: ET_DEFERRED_FUNCTION) is
--			-- Process child nodes, create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := Void
--			current_function := a_routine
--			process_function
--		end

	process_do_procedure (a_routine: ET_DO_PROCEDURE) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := a_routine
			current_function := Void
			current_creator := creator (current_procedure, universe)

			Precursor (a_routine)
			process_procedure
			current_creator := Void
		end

	process_do_function (a_routine: ET_DO_FUNCTION) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := Void
			current_function := a_routine

			Precursor (a_routine)
			process_function
		end

--	process_external_procedure (a_routine: ET_EXTERNAL_PROCEDURE) is
--			-- Create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := a_routine
--			current_function := Void
--			current_creator := creator

--			process_procedure
--			current_creator := Void
--		end

--	process_external_function (a_routine: ET_EXTERNAL_FUNCTION) is
--			-- Create and process entry and exit program points for `a_routine'
--		do
--			current_procedure := Void
--			current_function := a_routine
--			process_function
--		end

	process_once_procedure (a_routine: ET_ONCE_PROCEDURE) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := a_routine
			current_function := Void

			Precursor (a_routine)
			process_procedure
		end

	process_once_function (a_routine: ET_ONCE_FUNCTION) is
			-- Process child nodes, create and process entry and exit program points for `a_routine'
		do
			current_procedure := Void
			current_function := a_routine

			Precursor (a_routine)
			process_function
		end

	process_loop_instruction (an_instruction: ET_LOOP_INSTRUCTION) is
			-- Process child nodes, create and process loop program point for `an_instruction'
		local
			instr_count: INTEGER
			l: CI_LOOP
			internal_procedure: ET_INTERNAL_PROCEDURE
			internal_function: ET_INTERNAL_FUNCTION
		do
			instr_count := instruction_count + 1
			Precursor (an_instruction)

			check
				(current_class /= Void) and ((current_procedure /= Void) or (current_function /= Void))
					-- `client_class' was set in `process_class' before processing its child nodes
					-- `current_procedure' or `current_function' was set in `process_procedure' or `process_function'
			end

			internal_procedure ?= current_procedure
			internal_function ?= current_function
			if internal_procedure /= Void then
				create l.make (an_instruction, instr_count, internal_procedure, current_class, universe)
			elseif internal_function /= Void then
				create {CI_FUNCTION_LOOP} l.make (an_instruction, instr_count, internal_function, current_class, universe)
			end
			process_program_point (l)
		end

feature {NONE} -- Implementation
	current_class: ET_CLASS
			-- Class that is currently processed

	current_procedure: ET_PROCEDURE
			-- Procedure that is currently processed

	current_function: ET_FUNCTION
			-- Function that is currently processed

--	current_internal_routine: ET_INTERNAL_ROUTINE

	current_creator: ET_CREATOR
			-- Creator of `current_class' or its descendants, which lists `current_procedure'
			-- Void, if not a creation procedure

	process_procedure is
			-- Create and process entry and exit program points for `current_procedure'
		require
			current_class_exists: current_class /= Void
			current_procedure_exists: current_procedure /= Void
		local
			entry: CI_ROUTINE_POINT
			exit: CI_ROUTINE_EXIT
		do
			if current_creator /= Void then
				create {CI_CREATION_PROCEDURE_ENTRY} entry.make (current_procedure, current_creator.clients,
					current_class, universe)
			else
				create {CI_ROUTINE_ENTRY} entry.make (current_procedure, current_class, universe)
			end
			process_program_point (entry)

			create exit.make (current_procedure, current_class, universe)
			process_program_point (exit)
		end

	process_function is
			-- Create and process entry and exit program points for `current_function'
		require
			current_class_exists: current_class /= Void
			current_function_exists: current_function /= Void
		local
			entry: CI_ROUTINE_ENTRY
			exit: CI_FUNCTION_EXIT
		do
			create entry.make (current_function, current_class, universe)
			process_program_point (entry)

			create exit.make (current_function, current_class, universe)
			process_program_point (exit)
		end

	process_program_point (a_ppt: CI_PROGRAM_POINT) is
			-- Process `a_ppt'
		require
			a_ppt_exists: a_ppt /= Void
		do
			program_points.extend (a_ppt, printer.program_point_name (a_ppt))
		end

invariant
	program_points_exists: program_points /= Void
	printer_exists: printer /= Void
	can_not_process_procedure_and_function: (current_procedure = Void) or (current_function = Void)
end
