indexing
	description: "Objects that print static program point information and produce instructions to print program point state"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CI_PROGRAM_POINT_PRINTER

inherit
	CI_RUNTIME_OUTPUT_METACLIENT

feature -- Processing
	process (a_program_point: CI_PROGRAM_POINT) is
			-- Print static information about `a_program_point' and build instructions that print its dynamic state
		require
			a_program_point_exists: a_program_point /= Void
			a_program_point_variables_added: a_program_point.variables_added
			static_output_exists: static_output /= Void
			static_output_open: static_output.is_open_write or static_output.is_open_append
		deferred
		ensure
			state_printing_instructions_exists: state_printing_instructions /= Void
		end

feature -- Access
	state_printing_instructions: ET_COMPOUND
			-- State printing instructions for last processed program point

	extra_locals: ET_LOCAL_VARIABLE_LIST
			-- Additional locals that are used in `state_printing_instructions'

	static_output: FILE
			-- File for static information output

feature -- Setting
	set_static_output (an_output: like static_output) is
			-- Set `static_ouput' to `an_output'
		do
			static_output := an_output
		ensure
			static_output_set: static_output = an_output
		end

feature -- Output
	program_point_name (a_program_point: CI_PROGRAM_POINT): STRING is
			-- Program point name
		require
			a_program_point_exists: a_program_point /= Void
		deferred
		ensure
			result_exists: Result /= Void
		end

feature {CI_PROGRAM_POINT} -- Program point kinds
	entry_kind: STRING is
			-- Name of routine entry program points
			-- "Visitor" pattern: program point kind name depends on ppt kind (ppt type) as well as on printing format (printer type);
			--  new printing formats (other than Daikon) will more likely be added
		deferred
		end

	exit_kind: STRING is
			-- Name of routine exit program points
		deferred
		end

	stable_kind: STRING is
			-- Name of stable-time program points
		deferred
		end

	body_kind (position: INTEGER): STRING is
			-- Name of routine body program points with position `position'
		deferred
		ensure
			body_kind_exists: Result /= Void
		end

	loop_kind (position:  INTEGER): STRING is
			-- Name of loop program points with position `position'
		deferred
		ensure
			loop_kind_exists: Result /= Void
		end

invariant
	entry_kind_exists: entry_kind /= Void
	exit_kind_exists: exit_kind /= Void
	stable_kind_exists: stable_kind /= Void
end
