indexing
	description: "Program points at entry to specific routine (associated with preconditions)"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_ROUTINE_ENTRY

inherit
	CI_ROUTINE_POINT
		redefine
			suppressor,
			add_variables,
			instrument
		end

	CI_RUNTIME_OUTPUT_NAMES_CLIENT

create
	make

feature -- Access
	suppressor: CI_PROGRAM_POINT is
			-- Program point, who's assertions suppress assertions at current program point (Void if no suppressor)
		do
			if not is_secret (surrounding_routine) then
				Result := create {CI_STABLE_TIME}.make (surrounding_class, universe)
			end
		end

feature {CI_PROGRAM_POINT_PRINTER} -- Printing
	kind (a_visitor: CI_PROGRAM_POINT_PRINTER): STRING is
			-- Program point kind name
		do
			Result := a_visitor.entry_kind
		end

feature -- Variables
	add_variables is
			-- Find variables in scope and store them into `variables'
		do
			Precursor
			-- Add routine's clients to all variable's `clients'
			from
				variables.start
			until
				variables.after
			loop
				variables.item.clients.resize (variables.item.clients.count + surrounding_routine.clients.count)
				variables.item.clients.extend_first (surrounding_routine.clients)
				variables.forth
			end
			variables_added := True
		end

feature -- Assertions
	checked_assertions: ET_ASSERTIONS is
			-- Eiffel assertions, that are checked at this program point
		do
			Result := surrounding_routine.preconditions
		end

	instrument is
			-- Instrument program point by creating a check inside `surrounding_routine' with call to `state_printing_function'
		local
			internal_routine: ET_INTERNAL_ROUTINE
			check_instr: ET_CHECK_INSTRUCTION
		do
--			if checked_assertions /= Void then
--				checked_assertions.wipe_out
--			end
			internal_routine ?= surrounding_routine
			-- Works only for internal routine as present (more correctly is to call `state_printing_function' from check on client side)
			if internal_routine /= Void then
				create check_instr.make_with_capacity (1)
				check_instr.put_first (create {ET_CALL_EXPRESSION}.make (Void, state_printing_function_name, state_printing_function_actuals))
				internal_routine.compound.force_first (check_instr)
			end
		end

feature {NONE} -- Assertions
	create_assertions is
			-- Create empty `checked_assertions', if they don't exist
		local
			list: ET_PRECONDITIONS
		do
			if surrounding_routine.preconditions = Void then
				create list.make
				list.set_else_keyword (tokens.else_keyword)  -- `require else' is a safe default
				surrounding_routine.set_preconditions (list)
			end
		end

	process_original_assertions is
			-- Process original assertions according to settings while annotating this point
		local
			p: ET_ASSERTION
		do
			if Settings.keep_original_contracts = Settings.remove_mode then
				checked_assertions.wipe_out
			elseif Settings.keep_original_contracts = Settings.flatten_mode then
				p := tagged_flat_precondition (surrounding_routine, surrounding_class, universe)
				checked_assertions.wipe_out
				if p /= Void then
					checked_assertions.force_first (p)
				end
			end
		end

	state_printing_function_name: ET_FEATURE_NAME is
			-- Name of `state_printing_function' for this program point
		do
			create {ET_IDENTIFIER} Result.make (surrounding_class.lower_name + "_" +
				identifier_name (surrounding_routine.name).lower_name + "_pre_")
		end
end
