indexing
	description: "Names inside RUNTIME_OUTPUT and RUNTIME_OUTPUT_CLIENT classes.%
		% This class is singleton. To use it inherit from CI_RUNTIME_OUTPUT_NAMES_CLIENT"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_RUNTIME_OUTPUT_NAMES

inherit
	CI_SETTINGS_CLIENT

	CI_SHARED_TOKEN_CONSTANTS

create {CI_RUNTIME_OUTPUT_NAMES_CLIENT}
	default_create

feature -- Classes
	default_runtime_output_name: STRING is
			-- Default name of "RUNTIME_OUTPUT" class
		once
			Result := "RUNTIME_OUTPUT"
		end

	default_runtime_output_client_name: STRING is
			-- Default name of "RUNTIME_OUTPUT_CLIENT" class
		once
			Result := "RUNTIME_OUTPUT_CLIENT"
		end

	runtime_output_name: ET_CLASS_NAME
			-- Name of "RUNTIME_OUTPUT" class

	runtime_output_client_name: ET_CLASS_NAME
			-- Name of "RUNTIME_OUTPUT_CLIENT" class

	generate_runtime_output_name (a_universe: ET_UNIVERSE) is
			-- Generate name for "RUNTIME_OUTPUT" class that is unique within `a_universe'
		require
			a_universe_exists: a_universe /= Void
		do
			from
				create {ET_IDENTIFIER} runtime_output_name.make (default_runtime_output_name)
			until
				not a_universe.has_class (runtime_output_name)
			loop
				runtime_output_name.name.append_character ('_')
			end
		ensure
			runtime_output_name_exists: runtime_output_name /= Void
			runtime_output_name_unique: not a_universe.has_class (runtime_output_name)
		end

	generate_runtime_output_client_name (a_universe: ET_UNIVERSE) is
			-- Generate name for "RUNTIME_OUTPUT_CLIENT" class that is unique within `a_universe'
		require
			a_universe_exists: a_universe /= Void
		do
			if not Settings.is_any_changed then
				-- Separate "RUNTIME_OUTPUT_CLIENT" class will be generated
				from
					create {ET_IDENTIFIER} runtime_output_client_name.make (default_runtime_output_client_name)
				until
					not a_universe.has_class (runtime_output_client_name)
				loop
					runtime_output_client_name.name.append_character ('_')
				end
			else
				-- Features of "RUNTIME_OUTPUT_CLIENT" will be put to "ANY"
				runtime_output_client_name := tokens.any_class_name
			end
		ensure
			runtime_output_client_name_exists: runtime_output_client_name /= Void
			runtime_output_client_name_unique_or_any: not a_universe.has_class (runtime_output_client_name) or
				runtime_output_client_name.same_class_name (tokens.any_class_name)
		end

feature -- Features
	file_feature_name: ET_FEATURE_NAME is
			-- Name of "file" feature from "RUNTIME_OUTPUT"
		once
			create {ET_IDENTIFIER} Result.make ("file")
		end

	print_var_feature_name: ET_FEATURE_NAME is
			-- Name of "print_var" feature from "RUNTIME_OUTPUT"
		once
			create {ET_IDENTIFIER} Result.make ("print_var")
		end

	string_rep_feature_name: ET_FEATURE_NAME is
			-- Name of "string_rep" feature from "RUNTIME_OUTPUT"
		once
			create {ET_IDENTIFIER} Result.make ("string_rep")
		end

	trace_feature_name: ET_FEATURE_NAME is
			-- Name of "trace" feature from "RUNTIME_OUTPUT_CLIENT"
		once
			create {ET_IDENTIFIER} Result.make ("trace_")
		end

	called_feature_name (a_class_name: ET_CLASS_NAME; a_feature_name: ET_FEATURE_NAME): ET_IDENTIFIER is
			-- Name of "<class>_<feature>_called" feature from "RUNTIME_OUTPUT"
		require
			a_class_name_exists: a_class_name /= Void
			a_feature_name_exists: a_feature_name /= Void
		do
			create Result.make (a_class_name.lower_name + "_" + a_feature_name.lower_name + "_called")
		ensure
			result_exists: Result /= Void
		end

	set_called_feature_name (a_class_name: ET_CLASS_NAME; a_feature_name: ET_FEATURE_NAME): ET_FEATURE_NAME is
			-- Name of "set_<class>_<feature>_called" feature from "RUNTIME_OUTPUT"
		require
			a_class_name_exists: a_class_name /= Void
			a_feature_name_exists: a_feature_name /= Void
		do
			create {ET_IDENTIFIER} Result.make ("set_" + a_class_name.lower_name + "_" + a_feature_name.lower_name + "_called")
		ensure
			result_exists: Result /= Void
		end

feature -- Status report
	is_trace_feature_name (a_name: ET_FEATURE_NAME): BOOLEAN is
			-- Is `a_name' name of "trace" feature?
		require
			a_name_exists: a_name /= Void
		do
			Result := a_name.same_feature_name (trace_feature_name)
		end

feature -- Formal arguments
	name_argument: ET_IDENTIFIER is
			-- Name of "name" formal argument for "print_var"
		once
			create Result.make ("name")
		end

	value_argument: ET_IDENTIFIER is
			-- Name of "value" formal argument for "print_var"
		once
			create Result.make ("value")
		end

	modified_argument: ET_IDENTIFIER is
			-- Name of "modified" formal argument for "print_var"
		once
			create Result.make ("modified")
		end

	item_argument: ET_IDENTIFIER is
			-- Name of formal argument for "hash_code"
		once
			create Result.make ("item")
		end
invariant
	default_runtime_output_name_exists: default_runtime_output_name /= Void
	default_runtime_output_client_name_exists: default_runtime_output_client_name /= Void
	file_name_exists: file_feature_name /= Void
	print_var_name_exists: print_var_feature_name /= Void
	string_rep_feature_name_exists: string_rep_feature_name /= Void
	trace_name_exists: trace_feature_name /= Void
	name_argument_exists: name_argument /= Void
	value_argument_exists: value_argument /= Void
	modified_argument_exists: modified_argument /= Void
end
