indexing
	description: "Objects that generate, build and execute instrumented systems"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_INSTRUMENTED_SYSTEM

inherit
	CI_SETTINGS_CLIENT

	CI_ERROR_HANDLER_CLIENT

	CONF_ACCESS
		export {NONE} all end

	CONF_CONSTANTS
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM

	THREAD_CONTROL
		export {NONE} all end

	PROCESS_FACTORY
		export {NONE} all end

create
	make_null

feature -- Access
	instrumented_target_name, instrumented_cluster_name: STRING is "instrumented"
			-- Target name for instrumented system

	ise_generated_directory_name: STRING is "EIFGENs"
			-- Name of directory, where files generated by ise eiffel compiler are placed

	code_directory_name: STRING is
			-- Name of directory, where executable files are placed by ise eiffel compiler
		do
			if Settings.workbench_mode then
				Result := "W_code"
			else
				Result := "F_code"
			end
		end

	instrumented_executable_name: FILE_NAME is
			-- Name of executable file for instrumented system
		do
			create Result.make
			Result.set_directory (Settings.instrumented_system_path)
			Result.set_subdirectory (ise_generated_directory_name)
			Result.set_subdirectory (instrumented_target_name)
			Result.set_subdirectory (code_directory_name)
			Result.set_file_name (Settings.project_name)
			if not Settings.exe_extension.is_empty then
				Result.add_extension (Settings.exe_extension)
			end
		end

feature -- Basic Operations
	print_class (a_class: ET_CLASS; a_universe: ET_UNIVERSE) is
			-- Print code of `a_class', which appears in `a_universe'
		require
			a_class_exists: a_class /= Void
		local
			out_file: KI_TEXT_OUTPUT_FILE
			printer: CI_AST_PRETTY_PRINTER
		do
			out_file := file_system.new_output_file (Settings.instrumented_class_file_name (a_class.name))
			out_file.open_write
			if out_file.is_open_write then
				create printer.make (out_file, a_universe)
				a_class.process (printer)
				out_file.close
			else
				error_handler.report_error_message ("Can't create file " + Settings.instrumented_class_file_name (a_class.name))
			end
		end

	generate_configuration_file (a_system: CONF_SYSTEM) is
			-- Generate configuration file for instrumented system
			-- (`a_system' is original system configuration)
		local
			factory: CONF_PARSE_FACTORY
			app_target, instrumented_target: CONF_TARGET
			instrumented_cluster: CONF_OVERRIDE
			dir_loc: CONF_DIRECTORY_LOCATION
--			no_assertions,
			instrumented_assertions: CONF_ASSERTIONS
			libs: HASH_TABLE [CONF_LIBRARY, STRING]
			clusters: HASH_TABLE [CONF_CLUSTER, STRING]
			overrides: HASH_TABLE [CONF_OVERRIDE, STRING]
		do
			create factory
			app_target := a_system.application_target

				-- Add a new target for instrumented system
			instrumented_target := factory.new_target (instrumented_target_name, a_system)
			a_system.add_target (instrumented_target)
			instrumented_target.set_settings (app_target.settings)
			instrumented_target.add_setting (s_executable_name, Settings.project_name)
				-- Set the relative path to the original system location
			instrumented_target.add_setting (s_library_root, a_system.directory)

				-- Add override cluster with instrumented classes
			dir_loc := factory.new_location_from_path (Settings.instrumented_cluster_path, instrumented_target)
			instrumented_cluster := factory.new_override (instrumented_cluster_name, dir_loc, instrumented_target)
			instrumented_cluster.set_recursive (False)
--			instrumented_cluster.changeable_internal_options.set_assertions (instrumented_assertions)
			instrumented_target.add_override (instrumented_cluster)

				-- Set the root class
			instrumented_target.set_root (app_target.root)

				-- Set application assertion level
--			no_assertions := factory.new_assertions
--			instrumented_target.changeable_internal_options.set_assertions (no_assertions)
			instrumented_assertions := factory.new_assertions
--			instrumented_assertions.enable_precondition
--			instrumented_assertions.enable_postcondition
			instrumented_assertions.enable_invariant
			instrumented_assertions.enable_check
			instrumented_assertions.enable_loop
			instrumented_target.changeable_internal_options.set_assertions (instrumented_assertions)

				-- Copy all groups to the new target
				-- Change assertion level of libraries to check no assertions.
			from
				libs := app_target.libraries
				libs.start
			until
				libs.after
			loop
--				libs.item_for_iteration.changeable_internal_options.set_assertions (no_assertions)
				instrumented_target.add_library (libs.item_for_iteration)
				libs.forth
			end

			from
				clusters := app_target.clusters
				clusters.start
			until
				clusters.after
			loop
				instrumented_target.add_cluster (clusters.item_for_iteration)
				clusters.forth
			end

			from
				overrides := app_target.overrides
				overrides.start
			until
				overrides.after
			loop
				instrumented_target.add_override (overrides.item_for_iteration)
				overrides.forth
			end

				-- Set the new name and location of the generated ecf file
			a_system.set_file_name (Settings.instrumented_system_name)
			a_system.store
			if not a_system.store_successful then
				error_handler.report_configuration_storage_failed
			end
		end

	compile is
			-- Compile instrumented system
		local
			ise_eiffel_compiler: CI_ISE_EIFFEL_COMPILER
			line_reader: AUT_THREAD_SAFE_LINE_READER
		do
			create ise_eiffel_compiler.make
			if not Settings.workbench_mode then
				ise_eiffel_compiler.enable_finalize
			end
			create line_reader.make
			ise_eiffel_compiler.set_output_handler (agent line_reader.put_string)
			ise_eiffel_compiler.set_on_launch_failed (agent error_handler.report_launch_failed (
				ise_eiffel_compiler.eiffel_compiler_executable_name))

			from
				ise_eiffel_compiler.run (Settings.instrumented_system_path, Settings.instrumented_system_name,
					instrumented_target_name)
			until
				not ise_eiffel_compiler.is_running
			loop
				from
					line_reader.try_read_line
				until
					not line_reader.has_read_line
				loop
					error_handler.report_info_message (line_reader.last_line)
					line_reader.try_read_line
				end
				sleep (timeout)
			end
			if not ise_eiffel_compiler.was_successful then
				error_handler.report_instrumented_system_compilation_failed
			end
		end

	execute is
			-- Execute instrumented system
		local
			cmd_line: STRING
		do
			cmd_line := instrumented_executable_name.string
			if Settings.instrumented_system_arguments /= Void then
				cmd_line.append_character (' ')
				cmd_line.append (Settings.instrumented_system_arguments)
			end
			process := process_launcher_with_command_line (cmd_line, Settings.instrumented_system_path)
			process.enable_launch_in_new_process_group
			process.set_on_fail_launch_handler (agent error_handler.report_launch_failed (instrumented_executable_name))
			process.set_hidden (True)
			process.launch
			if process.launched then
				process.wait_for_exit
			end
			if process.has_exited and process.exit_code /= 0 then
				error_handler.report_instrumented_system_execution_failed
			end
		end

feature {NONE} -- Implementation
	timeout: INTEGER_64 is 1_000_000_000
			-- Timeout to wait for next compiler output (nanoseconds)

	process: PROCESS
			-- Process, in which instrumented system is launched
end
