indexing
	description: "Some functions for Eiffel code generation%
		% This class may be used as ancestor by classes needing its facilities"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	CI_CODE_GENERATION

inherit
	CI_SHARED_TOKEN_CONSTANTS

feature -- Compounds
	insert_instruction_to_front (a_compound: ET_COMPOUND; an_instruction: ET_INSTRUCTION) is
			-- Insert `an_instruction' at the beginning of `a_compound'
		require
			a_compound_exists: a_compound /= Void
			an_instrucntion_exists: an_instruction /= Void
		do
			a_compound.force_first (an_instruction)
		ensure
			one_more_element: a_compound.count = old a_compound.count + 1
			new_element_at_beginning: a_compound.first = an_instruction
		end

	insert_instruction_to_back (a_compound: ET_COMPOUND; an_instruction: ET_INSTRUCTION) is
			-- Insert `an_instruction' at the end of `a_compound'
		require
			a_compound_exists: a_compound /= Void
			an_instrucntion_exists: an_instruction /= Void
		do
			insert_compound_to_back (a_compound, single_instruction (an_instruction))
		ensure
			one_more_element: a_compound.count = old a_compound.count + 1
			new_element_at_end: a_compound.last = an_instruction
		end

	insert_instruction (a_compound: ET_COMPOUND; an_instruction: ET_INSTRUCTION; n: INTEGER) is
			-- Insert `an_instruction' into `a_compound' after item `n'
		require
			a_compound_exists: a_compound /= Void
			an_instrucntion_exists: an_instruction /= Void
			n_not_too_small: n >= 0
			n_not_too_large: n <= a_compound.count
		do
			insert_compound (a_compound, single_instruction (an_instruction), n)
		ensure
			one_more_element: a_compound.count = old a_compound.count + 1
			new_element_at_right_position: a_compound.item (n + 1) = an_instruction
		end

	insert_compound_to_front (a_compound, an_insertion: ET_COMPOUND) is
			-- Prepend `an_insertion' to `a_compound'
		require
			a_compound_exists: a_compound /= Void
			an_insertion_exists: an_insertion /= Void
		do
			insert_compound (a_compound, an_insertion, 0)
		ensure
			right_count: a_compound.count = old a_compound.count + an_insertion.count
		end

	insert_compound_to_back (a_compound, an_insertion: ET_COMPOUND) is
			-- Append `an_insertion' to `a_compound'
		require
			a_compound_exists: a_compound /= Void
			an_insertion_exists: an_insertion /= Void
		do
			insert_compound (a_compound, an_insertion, a_compound.count)
		ensure
			right_count: a_compound.count = old a_compound.count + an_insertion.count
		end

	insert_compound (a_compound, an_insertion: ET_COMPOUND; n: INTEGER) is
			-- Insert `an_insertion' to `a_compound' after item `n'
			-- Bad performance :(
		require
			a_compound_exists: a_compound /= Void
			an_insertion_exists: an_insertion /= Void
			n_not_too_small: n >= 0
			n_not_too_large: n <= a_compound.count
		local
			temp: ARRAY [ET_INSTRUCTION]
			i, k: INTEGER
		do
			-- Save first `n' items of `a_compound' to `temp' and remove them from `a_compound'
			create temp.make (1,n)
			k := a_compound.count - n
			from
				i := 1
			until
				a_compound.count = k
			loop
				temp [i] := a_compound.first
				a_compound.remove_first
				i := i + 1
			end

			-- Put `an_insertion' items to `a_compound'
			a_compound.resize (n + k + an_insertion.count)
			from
				i := an_insertion.count
			until
				i < 1
			loop
				a_compound.put_first (an_insertion.item (i))
				i := i - 1
			end

			-- Put first `n' items back to `a_compound'
			from
				i := n
			until
				i < 1
			loop
				a_compound.put_first (temp [i])
				i := i - 1
			end
		ensure
			right_count: a_compound.count = old a_compound.count + an_insertion.count
		end

feature -- Assertions
	joined (an_assertions: ET_ASSERTIONS; an_operator: ET_OPERATOR): ET_EXPRESSION is
			-- Expressions of `an_assertions' joined by `an_operator'
		require
			an_assertions_not_empty: an_assertions /= Void and then not an_assertions.is_empty
			an_operator_exists: an_operator /= Void
		local
			i: INTEGER
		do
			from
				Result := parenthesized_for_binary (an_assertions.first.assertion.expression)
				i := 2
			until
				i > an_assertions.count
			loop
				create {ET_INFIX_EXPRESSION} Result.make (Result,
					an_operator,
					parenthesized_for_binary (an_assertions.item (i).assertion.expression))
				i := i + 1
			end
		ensure
			result_exists: Result /= Void
		end

	all_precursors (a_feature: ET_FEATURE): LINKED_SET [ET_FEATURE] is
			-- All precursors of `a_feature' in `a_universe'
		require
			a_feature_exists: a_feature /= Void
		local
			i: INTEGER
		do
			create Result.make
			if a_feature.first_precursor /= Void then
				Result.append (all_precursors (a_feature.first_precursor))
			end
			if a_feature.other_precursors /= Void then
				from i := 1	until i > a_feature.other_precursors.count loop
					Result.append (all_precursors (a_feature.other_precursors.item (i)))
					i := i + 1
				end
			end
			if a_feature.first_precursor /= Void then
				Result.extend (a_feature.first_precursor)
			end
			if a_feature.other_precursors /= Void then
				from i := 1	until i > a_feature.other_precursors.count loop
					Result.extend (a_feature.other_precursors.item (i))
					i := i + 1
				end
			end
		ensure
			result_exists: Result /= Void
			first_precursor_present: a_feature.first_precursor /= Void implies Result.has (a_feature.first_precursor)
			feature_not_included: not Result.has (a_feature)
		end

	guard (a_preconditions: ET_ASSERTIONS): ET_IF_INSTRUCTION is
			-- Create if instruction with conjunction of `a_preconditions' as condition
		require
			a_preconditions_exists: a_preconditions /= Void
		local
			expression: ET_EXPRESSION
			conditional: ET_KEYWORD_EXPRESSION
			then_compound, else_compound: ET_COMPOUND
		do
			if a_preconditions.is_empty then
				expression := tokens.true_keyword
			else
				expression := joined (a_preconditions, tokens.and_then_operator)
			end
			create conditional.make (tokens.if_keyword, expression)
			create then_compound.make
			then_compound.set_keyword (tokens.then_keyword)
			create else_compound.make
			else_compound.set_keyword (tokens.else_keyword)
			create Result.make (conditional, then_compound)
			Result.set_else_compound (else_compound)
		ensure
			result_exists: Result /= Void
		end

	exists_condition (an_expression: ET_EXPRESSION): ET_EQUALITY_EXPRESSION is
			-- Assetion that `an_expression' exists (`an_expression /= Void')
		require
			an_expression_exists: an_expression /= Void
		local
			ne: ET_SYMBOL
		do
			create ne.make_not_equal
			create Result.make (parenthesized_for_binary (an_expression), ne, tokens.void_keyword)
		ensure
			result_exists: Result /= Void
		end

	flat_precondition (a_feature: ET_FEATURE; a_class: ET_CLASS; a_universe: ET_UNIVERSE): ET_EXPRESSION is
			-- Flat form of precondition of `a_feature' (may be Void)
		require
			a_feature_exists: a_feature /= Void
		local
			precursors: LINKED_SET [ET_FEATURE]
			p: ET_FEATURE
			all_preconditions: ET_PRECONDITIONS
			disjunct: ET_EXPRESSION
			renamer: CI_FEATURE_RENAMER
		do
			create all_preconditions.make
			if a_feature.preconditions /= Void and then not a_feature.preconditions.is_empty then
				all_preconditions.force_first (joined (a_feature.preconditions, tokens.and_then_operator))
			end
			precursors := all_precursors (a_feature)
			from
				precursors.finish
			until
				precursors.before
			loop
				p := precursors.item
				if p.preconditions /= Void and then not p.preconditions.is_empty then
					disjunct := joined (p.preconditions, tokens.and_then_operator)
					create renamer.make (a_feature, p, p.implementation_class, a_class, a_universe)
					disjunct.process (renamer)
					disjunct ?= renamer.modified_node
					all_preconditions.force_first (disjunct)
				end
				precursors.back
			end

			if not all_preconditions.is_empty then
				Result := joined (all_preconditions, tokens.or_else_operator)
			end
		ensure
			exists_if_immediate_preconditions: a_feature.preconditions /= Void and then not a_feature.preconditions.is_empty implies
				Result /= Void
		end

	tagged_flat_precondition (a_feature: ET_FEATURE; a_class: ET_CLASS; a_universe: ET_UNIVERSE): ET_TAGGED_ASSERTION is
			-- Flat form of precondition of `a_feature' with a tag (may be Void)
		local
			expr: ET_EXPRESSION
		do
			expr := flat_precondition (a_feature, a_class, a_universe)
			if expr /= Void then
				create Result.make (create {ET_IDENTIFIER}.make ("flattened"))
				Result.set_expression (expr)
			end
		end

	flat_postcondition (a_feature: ET_FEATURE; a_class: ET_CLASS; a_universe: ET_UNIVERSE): ET_POSTCONDITIONS is
			-- Flat form of postcondition of `a_feature' (may be Void)
		require
			a_feature_exists: a_feature /= Void
		local
			precursors: LINKED_SET [ET_FEATURE]
			p: ET_FEATURE
			i: INTEGER
			a: ET_ASSERTION
			renamer: CI_FEATURE_RENAMER
		do
			create Result.make
			prepend_assertions (Result, a_feature.postconditions)
			precursors := all_precursors (a_feature)
			from
				precursors.finish
			until
				precursors.before
			loop
				p := precursors.item
				if p.postconditions /= Void then
					from
						i := p.postconditions.count
					until
						i < 1
					loop
						a := p.postconditions.assertion (i)
						create renamer.make (a_feature, p, p.implementation_class, a_class, a_universe)
						a.process (renamer)
						a ?= renamer.modified_node
						check a /= Void end
						Result.force_first (a)
						i := i - 1
					end
				end
				precursors.back
			end
		ensure
			result_exists: Result /= Void
		end

	flat_invariants (a_class: ET_CLASS; a_universe: ET_UNIVERSE): ET_INVARIANTS is
			-- Flat form of class invariants for `a_class'
		local
			ancestor: ET_CLASS
			a: ET_ASSERTION
			renamer: CI_FEATURE_RENAMER
			i, j: INTEGER
		do
			create Result.make (a_class)
			prepend_assertions (Result, a_class.invariants)
			from
				i := 1
			until
				i > a_class.ancestors.count
			loop
				ancestor := a_class.ancestors.item (i).direct_base_class (a_universe)
				if ancestor.invariants /= Void then
					create renamer.make (Void, Void, ancestor, a_class, a_universe)
					from
						j := ancestor.invariants.count
					until
						j < 1
					loop
						a := ancestor.invariants.assertion (j)
						a.process (renamer)
						a ?= renamer.modified_node
						check a /= Void end
						Result.force_first (a)
						j := j - 1
					end
				end
				i := i + 1
			end
		end

feature -- Expressions
	manifest_string (a_literal: STRING): ET_REGULAR_MANIFEST_STRING is
			-- Regular manifest string with value equal to `a_literal'
		require
			a_literal_exists: a_literal /= Void
		do
			create Result.make (a_literal)
		ensure
			result_exists: Result /= Void
		end

	manifest_integer (a_literal: STRING): ET_REGULAR_INTEGER_CONSTANT is
			-- Regular integer constant represented by `a_literal'
		require
			a_literal_exists: a_literal /= Void
		do
			create Result.make (a_literal)
		ensure
			result_exists: Result /= Void
		end

	parenthesized (an_expression: ET_EXPRESSION): ET_PARENTHESIZED_EXPRESSION is
			-- Parenthesized version of `an_expression'
		require
			an_expression_exists: an_expression /= Void
		do
			create Result.make (an_expression)
		ensure
			result_exists: Result /= Void
		end

	parenthesized_for_dot (an_expression: ET_EXPRESSION): ET_EXPRESSION is
			-- Parenthesized if weaker then `.'
		require
			an_expression_exists: an_expression /= Void
		do
			if is_constant (an_expression) or is_unary (an_expression) or is_binary (an_expression) then
				Result := parenthesized (an_expression)
			else
				Result := an_expression
			end
		ensure
			result_exists: Result /= Void
		end

	parenthesized_for_binary (an_expression: ET_EXPRESSION): ET_EXPRESSION is
			-- Parenthesized if may be weaker then some binary
		require
			an_expression_exists: an_expression /= Void
		do
			if is_binary (an_expression) then
				Result := parenthesized (an_expression)
			else
				Result := an_expression
			end
		ensure
			result_exists: Result /= Void
		end

	infix_operator (a_name: ET_CALL_NAME): ET_OPERATOR is
			-- Infix operator that corresponds to `a_name'
		require
			a_name_exists: a_name /= Void
			a_name_is_infix: a_name.is_infix
		local
			symbol_operator: ET_SYMBOL_OPERATOR
		do
			if a_name.is_infix_and then
				create {ET_KEYWORD_OPERATOR} Result.make_and
			elseif a_name.is_infix_and_then then
				create {ET_INFIX_AND_THEN_OPERATOR} Result.make
			elseif a_name.is_infix_div then
				create {ET_SYMBOL_OPERATOR} Result.make_div
			elseif a_name.is_infix_divide then
				create {ET_SYMBOL_OPERATOR} Result.make_divide
			elseif a_name.is_infix_freeop then
				create {ET_FREE_OPERATOR} Result.make_infix (a_name.name)
			elseif a_name.is_infix_ge then
				create {ET_SYMBOL_OPERATOR} Result.make_ge
			elseif a_name.is_infix_gt then
				create {ET_SYMBOL_OPERATOR} Result.make_gt
			elseif a_name.is_infix_implies then
				create {ET_KEYWORD_OPERATOR} Result.make_implies
			elseif a_name.is_infix_le then
				create {ET_SYMBOL_OPERATOR} Result.make_le
			elseif a_name.is_infix_lt then
				create {ET_SYMBOL_OPERATOR} Result.make_lt
			elseif a_name.is_infix_minus then
				create symbol_operator.make_minus
				symbol_operator.set_infix_minus
				Result := symbol_operator
			elseif a_name.is_infix_mod then
				create {ET_SYMBOL_OPERATOR} Result.make_mod
			elseif a_name.is_infix_or then
				create {ET_KEYWORD_OPERATOR} Result.make_or
			elseif a_name.is_infix_or_else then
				create {ET_INFIX_OR_ELSE_OPERATOR} Result.make
			elseif a_name.is_infix_plus then
				create symbol_operator.make_plus
				symbol_operator.set_infix_plus
				Result := symbol_operator
			elseif a_name.is_infix_power then
				create {ET_SYMBOL_OPERATOR} Result.make_power
			elseif a_name.is_infix_times then
				create {ET_SYMBOL_OPERATOR} Result.make_times
			elseif a_name.is_infix_xor then
				create {ET_KEYWORD_OPERATOR} Result.make_xor
			end
		end

	prefix_operator (a_name: ET_CALL_NAME): ET_OPERATOR is
			-- Prefix operator that corresponds to `a_name'
		require
			a_name_exists: a_name /= Void
			a_name_is_prefix: a_name.is_prefix
		local
			symbol_operator: ET_SYMBOL_OPERATOR
		do
			if a_name.is_prefix_freeop then
				create {ET_FREE_OPERATOR} Result.make_prefix (a_name.name)
			elseif a_name.is_prefix_minus then
				create symbol_operator.make_minus
				symbol_operator.set_prefix_minus
				Result := symbol_operator
			elseif a_name.is_prefix_not then
				create {ET_KEYWORD_OPERATOR} Result.make_not
			elseif a_name.is_prefix_plus then
				create symbol_operator.make_plus
				symbol_operator.set_prefix_plus
				Result := symbol_operator
			end
		end

	bracket_argument_list (a_list: ET_ACTUAL_ARGUMENT_LIST): ET_BRACKET_ARGUMENT_LIST is
			-- Bracket argument list made from `a_list'
		local
			i: INTEGER
		do
			if a_list /= Void then
				create Result.make_with_capacity (a_list.count)
				from
					i := a_list.count
				until
					i < 1
				loop
					Result.put_first (a_list.item (i))
					i := i - 1
				end
			end
		end

	argument_list (a_list: ET_BRACKET_ARGUMENT_LIST): ET_ACTUAL_ARGUMENT_LIST is
			-- Bracket argument list made from `a_list'
		local
			i: INTEGER
		do
			if a_list /= Void then
				create Result.make_with_capacity (a_list.count)
				from
					i := a_list.count
				until
					i < 1
				loop
					Result.put_first (a_list.item (i))
					i := i - 1
				end
			end
		end

	is_unary (an_expression: ET_EXPRESSION): BOOLEAN is
			-- Is `an_expression' unary (or address) expression?
		local
			u: ET_UNARY_EXPRESSION
			a: ET_ADDRESS_EXPRESSION
		do
			u ?= an_expression
			a ?= an_expression
			Result := u /= Void or a/= Void
		end

	is_binary (an_expression: ET_EXPRESSION): BOOLEAN is
			-- Is `an_expression' binary?
		local
			b: ET_BINARY_EXPRESSION
		do
			b ?= an_expression
			Result := b /= Void
		end

	is_constant (an_expression: ET_EXPRESSION): BOOLEAN is
			-- Is `an_expression' a manifest constant?
		local
			const: ET_CONSTANT
		do
			const ?= an_expression
			Result := const /= Void
		end

	corrupted (a_name: STRING): ET_IDENTIFIER is
			-- Identifier with name which is corrupted form of `a_name'
		do
			create Result.make (a_name + "_")
		end

feature -- Class types
	type_from_nongeneric_class (a_class: ET_CLASS): ET_CLASS_TYPE is
			-- Class type corresponding to `a_class'
		require
			a_class_exists: a_class /= Void
			a_class_nongeneric: not a_class.is_generic
		do
			create Result.make (Void, a_class.name, a_class)
		ensure
			same_name: Result.name = a_class.name
		end

	type_from_parameter (a_parameter: ET_FORMAL_PARAMETER): ET_FORMAL_PARAMETER_TYPE is
			-- Formal parameter type that corresponds to `a_parameter'
		do
			create Result.make (a_parameter.name, a_parameter.index)
		end

	good_actual_parameter (a_parameter: ET_ACTUAL_PARAMETER): ET_ACTUAL_PARAMETER is
			-- Replaces `a_parameter' with formal parameter type if it is a formal parameter
		local
			formal: ET_FORMAL_PARAMETER
		do
			formal ?= a_parameter
			if formal /= Void then
				Result := type_from_parameter (formal)
			else
				Result := a_parameter
			end
		end

	good_generic_type (a_type: ET_GENERIC_CLASS_TYPE; a_class: ET_CLASS): ET_GENERIC_CLASS_TYPE is
			-- Generic class type made from `a_type', which is based on `a_class', by replacing each actual with `good_actual_parameter'
		local
			i: INTEGER
			actuals: ET_ACTUAL_PARAMETER_LIST
		do
			create actuals.make_with_capacity (a_type.actual_parameters.count)
			from
				i := a_type.actual_parameters.count
			until
				i < 1
			loop
				actuals.put_first (good_actual_parameter (a_type.actual_parameters.actual_parameter (i)))
				i := i - 1
			end
			create {ET_GENERIC_CLASS_TYPE} Result.make (a_type.type_mark, a_type.name, actuals, a_class)
		end

	type_from_class (a_class: ET_CLASS; a_context: ET_TYPE_CONTEXT; a_universe: ET_UNIVERSE): ET_CLASS_TYPE is
			-- Class type corresponding to `a_class' that may be generic
		require
			a_class_exists: a_class /= Void
			a_context_exists: a_context /= Void
			a_universe_exists: a_universe /= Void
		local
			generic_type: ET_GENERIC_CLASS_TYPE
		do
			if a_class.is_generic then
				create generic_type.make (Void, a_class.name, a_class.formal_parameters, a_class)
				Result := good_generic_type (generic_type, a_class)
			else
				Result := type_from_nongeneric_class (a_class)
			end
		ensure
			same_name: Result.name = a_class.name
		end

	good_type (a_type: ET_AST_NODE; a_context: ET_TYPE_CONTEXT; a_universe: ET_UNIVERSE): ET_AST_NODE is
			-- Not a class, formal parameter, or generic type with formals instead of actuals
		local
			c: ET_CLASS
			f: ET_FORMAL_PARAMETER
			g: ET_GENERIC_CLASS_TYPE
		do
			c ?= a_type
			if c /= Void then
				Result := type_from_class (c, a_context, a_universe)
			else
				f ?= a_type
				if f /= Void then
					Result := type_from_parameter (f)
				else
					g ?= a_type
					if g /= Void then
						Result := good_generic_type (g, a_universe.eiffel_class (g.name))
					else
						Result := a_type
					end
				end
			end
		end

feature -- Features
	seeded_feature (a_class: ET_CLASS; a_seed: INTEGER): ET_FEATURE is
			-- Feature from `a_class' with seed `a_seed'
		require
			a_class_exists: a_class /= Void
		do
			Result := a_class.seeded_query (a_seed)
			if Result = Void then
				Result := a_class.seeded_procedure (a_seed)
			end
		ensure
			query_or_procedure: Result = Void or Result = a_class.seeded_query (a_seed) or Result = a_class.seeded_procedure (a_seed)
		end

	single_actual (an_argument: ET_EXPRESSION): ET_ACTUAL_ARGUMENT_LIST is
			-- One argument list made from `an_argument'
		require
			an_argument_exists: an_argument /= Void
		do
			create Result.make_with_capacity (1)
			Result.put_first (an_argument)
		ensure
			has_an_argument: Result.first = an_argument
		end

	single_formal (an_argument: ET_FORMAL_ARGUMENT): ET_FORMAL_ARGUMENT_LIST is
			-- One argument list made from `an_argument'
		require
			an_argument_exists: an_argument /= Void
		do
			create Result.make_with_capacity (1)
			Result.put_first (an_argument)
		ensure
			has_an_argument: Result.first = an_argument
		end

	single_local (an_entity: ET_LOCAL_VARIABLE): ET_LOCAL_VARIABLE_LIST is
			-- One entity list made from `an_entity'
		require
			an_entity_exists: an_entity /= Void
		do
			create Result.make_with_capacity (1)
			Result.put_first (an_entity)
		ensure
			has_an_entity: Result.first = an_entity
		end

	single_instruction (an_instruction: ET_INSTRUCTION): ET_COMPOUND is
			-- One instruction compound made from `an_instruction'
		require
			an_instruction_exists: an_instruction /= Void
		do
			create Result.make_with_capacity (1)
			Result.put_first (an_instruction)
		ensure
			has_an_instruction: Result.first = an_instruction
		end

	clients_none: ET_CLIENTS is
			-- Client list that consists of NONE class only
		do
			create Result.make_with_capacity (1)
			Result.put_first (tokens.none_class_name)
			Result.set_right_brace (create {ET_SYMBOL}.make_right_brace)
		end

	clients_any: ET_CLIENTS is
			-- Client list that consists of ANY class only
		do
			create Result.make_with_capacity (1)
			Result.put_first (tokens.any_class_name)
			Result.set_right_brace (create {ET_SYMBOL}.make_right_brace)
		end

	is_secret (a_feature: ET_FEATURE): BOOLEAN is
			-- Is `a_feature' exported to NONE?
		require
			a_feature_exists: a_feature /= Void
		do
			Result := a_feature.clients /= Void and then a_feature.clients.is_none
		end

	is_exported_creation (a_feature: ET_FEATURE; a_universe: ET_UNIVERSE): BOOLEAN is
			-- Is `a_feature' exported creation procedure in `a_universe'?
		require
			a_feature_exists: a_feature /= Void
			a_universe_exists: a_universe /= Void
		local
			proc: ET_PROCEDURE
			cr: ET_CREATOR
		do
			proc ?= a_feature
			if proc /= Void then
				cr := creator (proc, a_universe)
				Result := cr /= Void and then (cr.clients = Void or else not cr.clients.is_none)
			end
		end

	creator (a_procedure: ET_PROCEDURE; a_universe: ET_UNIVERSE): ET_CREATOR is
			-- Creator in which `a_procedure' appears (in its implementation class or its descendants)
			-- Void, if it is not a creation procedure
		require
			a_procedure_exists: a_procedure /= Void
			a_universe_exists: a_universe /= Void
		local
			descendants: DS_ARRAYED_LIST [ET_CLASS]
		do
			-- TODO: Should take all creators and join their clients
			-- Search in current class
			Result := creator_in_class (a_procedure.first_seed, a_procedure.implementation_class)
			-- Check if `a_procedure' is `default-create'
			if Result = Void and (a_procedure.first_seed = a_universe.default_create_seed or
				a_procedure.other_seeds /= Void and then a_procedure.other_seeds.has (a_universe.default_create_seed)) then
				create Result.make_with_capacity (clients_any, 1)
				Result.put_first (a_procedure.name)
			end
			-- Search in descendants
			if Result = Void then
				descendants := a_procedure.implementation_class.descendants (a_universe)
				from
					descendants.start
				until
					descendants.after
				loop
					Result := creator_in_class (a_procedure.first_seed, descendants.item_for_iteration)
					if Result /= Void then
						descendants.finish
					end
					descendants.forth
				end
			end
		end

feature -- Search
	local_by_name (a_list: ET_LOCAL_VARIABLE_LIST; an_item: ET_LOCAL_NAME): ET_LOCAL_VARIABLE is
			-- Item of `a_list' with name `an_item'?
			-- Void if no such item
		require
			an_item_exists: an_item /= Void
		local
			i: INTEGER
		do
			if a_list /= Void then
				from
					i := 1
				until
					i > a_list.count
				loop
					if a_list.item (i).name.same_identifier (an_item.identifier) then
						Result := a_list.item (i).local_variable
						i := a_list.count
					end
					i := i + 1
				end
			end
		end

	formal_by_name (a_list: ET_FORMAL_ARGUMENT_LIST; an_item: ET_ARGUMENT_NAME): ET_FORMAL_ARGUMENT is
			-- Item of `a_list' with name `an_item'?
			-- Void if no such item
		require
			an_item_exists: an_item /= Void
		local
			i: INTEGER
		do
			if a_list /= Void then
				from
					i := 1
				until
					i > a_list.count
				loop
					if a_list.item (i).name.same_identifier (an_item.identifier) then
						Result := a_list.item (i).formal_argument
						i := a_list.count
					end
					i := i + 1
				end
			end
		end

	precursor_in_class (a_feature: ET_FEATURE; a_class: ET_CLASS): ET_FEATURE is
			-- Precursor of `a_feature' in class `a_class'
		local
			i: INTEGER
			p: ET_FEATURE
		do
			p := a_feature.first_precursor
			if p /= Void then
				Result := a_class.seeded_procedure (p.first_seed)
				if Result = Void then
					Result := a_class.seeded_query (p.first_seed)
				end
			end
			if a_feature.other_precursors /= Void then
				from
					i := a_feature.other_precursors.count
				until
					i < 1 or Result /= Void
				loop
					p := a_feature.other_precursors.item (i)
					Result := a_class.seeded_procedure (p.first_seed)
					if Result = Void then
						Result := a_class.seeded_query (p.first_seed)
					end
					i := i - 1
				end
			end
		end

feature -- Miscellaneous
	class_names_twin (a_list: ET_CLASS_NAME_LIST): ET_CLASS_NAME_LIST is
			-- Copy of `a_list'
		local
			i: INTEGER
		do
			if a_list /= Void then
				create Result.make_with_capacity (a_list.count)
				from
					i := a_list.count
				until
					i < 1
				loop
					Result.put_first (a_list.item (i))
					i := i - 1
				end
			end
		end

	formals_twin (a_formals: ET_FORMAL_ARGUMENT_LIST): ET_FORMAL_ARGUMENT_LIST is
			-- Copy of `a_formals'
		local
			i: INTEGER
		do
			if a_formals /= Void then
				create Result.make_with_capacity (a_formals.count)
				from
					i := a_formals.count
				until
					i < 1
				loop
					Result.put_first (a_formals.item (i))
					i := i - 1
				end
			end
		end

	locals_twin (a_locals: ET_LOCAL_VARIABLE_LIST): ET_LOCAL_VARIABLE_LIST is
			-- Copy of `a_locals'
		local
			i: INTEGER
		do
			if a_locals /= Void then
				create Result.make_with_capacity (a_locals.count)
				from
					i := a_locals.count
				until
					i < 1
				loop
					Result.put_first (a_locals.item (i))
					i := i - 1
				end
			end
		end

	prepend_assertions (a_list, a_head: ET_ASSERTIONS) is
			-- Prepend `a_head' to `a_list'
		require
			a_list_exists: a_list /= Void
		local
			i: INTEGER
		do
			if a_head /= Void then
				from
					i := a_head.count
				until
					i < 1
				loop
					a_list.force_first (a_head.item (i))
					-- We don't need comments
					a_list.first.last_leaf.set_break (Void)
					i := i - 1
				end
			end
		ensure
			a_head_added: a_head /= Void implies a_list.count = old a_list.count + a_head.count
		end

	escape_sequences: ARRAY [ET_SPECIAL_MANIFEST_STRING] is
			-- Widely used escape sequences
		once
			create Result.make (1, 6)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%B", "%B"), 1)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%F", "%F"), 2)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%N", "%N"), 3)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%R", "%R"), 4)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%T", "%T"), 5)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%U", "%U"), 6)
		end

	double_escape_sequences: ARRAY [ET_SPECIAL_MANIFEST_STRING] is
			-- Double escaped sequences from `escape_sequences'
		once
			create Result.make (1, 6)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%%%B", "%%B"), 1)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%%%F", "%%F"), 2)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%%%N", "%%N"), 3)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%%%R", "%%R"), 4)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%%%T", "%%T"), 5)
			Result.put (create {ET_SPECIAL_MANIFEST_STRING}.make ("%%%%U", "%%U"), 6)
		end

	double_quote_escaped (a_str: STRING): STRING is
			-- `a_str' with "%%" inserted before all double quotes
		do
			Result := a_str.string
			Result.replace_substring_all ("%"", "%%%"")
		end

	add_comment (a_comment: STRING; a_node: ET_AST_NODE) is
			-- Add comment with text `a_comment' to `a_node'
		require
			a_comment_exists: a_comment /= Void
			a_node_exists: a_node /= Void
		do
			a_node.last_leaf.set_break (create {ET_BREAK}.make (tokens.comment_symbol_name + a_comment + "%N"))
		ensure
			comment_added: a_node.last_leaf.break.text.has_substring (a_comment)
		end

feature {NONE} -- Implementation
	creator_in_class (a_seed: INTEGER; a_class: ET_CLASS): ET_CREATOR is
			-- Creator of `a_class' in which procedure with seed `a_seed' appears
			-- Void, if it isn't creation procedure of `a_class'
		require
			a_class_exists: a_class /= Void
		local
			name: ET_FEATURE_NAME
			i: INTEGER
			item: ET_CREATOR
		do
			if a_class.creators /= Void then
				name := a_class.seeded_procedure (a_seed).name
				from
					i := 1
				until
					i > a_class.creators.count
				loop
					item := a_class.creators.item (i)
					if item.has_feature_name (name) then
						Result := item
						i := a_class.creators.count
					end
					i := i + 1
				end
			end
		end

invariant
	clients_any_exists: clients_any /= Void
	clients_none_exists: clients_none /= Void
	escape_sequences_exists: escape_sequences /= Void
	double_escape_sequences_exists: double_escape_sequences /= Void
	same_lower: escape_sequences.lower = double_escape_sequences.lower
	same_upper: escape_sequences.upper = double_escape_sequences.upper
end
